@@ -65,21 +65,14 @@ import six
 from six.moves import queue
 from six.moves import urllib
 
-try:
-    import pkg_resources
-except ImportError:
-    pass
-
-from . import errors
+from . import errors, __version__
 from .workers import threadpool
 from .makefile import MakeFile
 
 
 __all__ = ['HTTPRequest', 'HTTPConnection', 'HTTPServer',
            'SizeCheckWrapper', 'KnownLengthRFile', 'ChunkedRFile',
-           'Gateway',
-           'get_ssl_adapter_class',
-]
+           'Gateway', 'get_ssl_adapter_class']
 
 
 if 'win' in sys.platform and hasattr(socket, 'AF_INET6'):
@@ -89,17 +82,9 @@ if 'win' in sys.platform and hasattr(socket, 'AF_INET6'):
         socket.IPV6_V6ONLY = 27
 
 
-try:
-    cp_version = pkg_resources.require('cherrypy')[0].version
-except Exception:
-    cp_version = 'unknown'
-
-
 if six.PY3:
     def ntob(n, encoding='ISO-8859-1'):
-        """Return the given native string as a byte string in the given
-        encoding.
-        """
+        """Return the given native string as a byte string in the given encoding."""
         # In Python 3, the native string type is unicode
         return n.encode(encoding)
 
@@ -107,9 +92,7 @@ if six.PY3:
         return b.decode(encoding)
 else:
     def ntob(n, encoding='ISO-8859-1'):
-        """Return the given native string as a byte string in the given
-        encoding.
-        """
+        """Return the given native string as a byte string in the given encoding."""
         # In Python 2, the native string type is bytes. Assume it's already
         # in the given encoding, which for ISO-8859-1 is almost always what
         # was intended.
@@ -198,7 +181,6 @@ def read_headers(rfile, hdict=None):
 
 
 class SizeCheckWrapper(object):
-
     """Wraps a file-like object, raising MaxSizeExceeded if too large."""
 
     def __init__(self, rfile, maxlen):
@@ -268,7 +250,6 @@ class SizeCheckWrapper(object):
 
 
 class KnownLengthRFile(object):
-
     """Wraps a file-like object, returning an empty string when exhausted."""
 
     def __init__(self, rfile, content_length):
@@ -325,7 +306,6 @@ class KnownLengthRFile(object):
 
 
 class ChunkedRFile(object):
-
     """Wraps a file-like object, returning an empty string when exhausted.
 
     This class is intended to provide a conforming wsgi.input value for
@@ -363,7 +343,7 @@ class ChunkedRFile(object):
             self.closed = True
             return
 
-##            if line: chunk_extension = line[0]
+#            if line: chunk_extension = line[0]
 
         if self.maxlen and self.bytes_read + chunk_size > self.maxlen:
             raise IOError('Request Entity Too Large')
@@ -467,7 +447,6 @@ class ChunkedRFile(object):
 
 
 class HTTPRequest(object):
-
     """An HTTP Request (and response).
 
     A single HTTP connection may consist of multiple request/response pairs.
@@ -657,7 +636,6 @@ class HTTPRequest(object):
 
     def read_request_headers(self):
         """Read self.rfile into self.inheaders. Return success."""
-
         # then all the http headers
         try:
             read_headers(self.rfile, self.inheaders)
@@ -862,8 +840,7 @@ class HTTPRequest(object):
             if status < 200 or status in (204, 205, 304):
                 pass
             else:
-                if (self.response_protocol == 'HTTP/1.1'
-                        and self.method != b'HEAD'):
+                if self.response_protocol == 'HTTP/1.1' and self.method != b'HEAD':
                     # Use the chunked transfer-coding
                     self.chunked_write = True
                     self.outheaders.append((b'Transfer-Encoding', b'chunked'))
@@ -919,7 +896,6 @@ class HTTPRequest(object):
 
 
 class HTTPConnection(object):
-
     """An HTTP connection (active socket).
 
     server: the Server object which received this connection.
@@ -1049,7 +1025,8 @@ class HTTPConnection(object):
             pass
 
     def _close_kernel_socket(self):
-        """
+        """Close kernel socket in outdated Python versions.
+
         On old Python versions,
         Python's socket module does NOT call close on the kernel
         socket when you call socket.close(). We do so manually here
@@ -1093,7 +1070,6 @@ else:
 
 
 class HTTPServer(object):
-
     """An HTTP server."""
 
     _bind_addr = '127.0.0.1'
@@ -1106,8 +1082,7 @@ class HTTPServer(object):
     """The minimum number of worker threads to create (default 10)."""
 
     maxthreads = None
-    """The maximum number of worker threads to create (default -1 = no limit).
-    """
+    """The maximum number of worker threads to create (default -1 = no limit)."""
 
     server_name = None
     """The name of the server; defaults to socket.gethostname()."""
@@ -1119,29 +1094,25 @@ class HTTPServer(object):
     features used in the response."""
 
     request_queue_size = 5
-    """The 'backlog' arg to socket.listen(); max queued connections
-    (default 5).
-    """
+    """The 'backlog' arg to socket.listen(); max queued connections (default 5)."""
 
     shutdown_timeout = 5
-    """The total time, in seconds, to wait for worker threads to cleanly exit.
-    """
+    """The total time, in seconds, to wait for worker threads to cleanly exit."""
 
     timeout = 10
     """The timeout in seconds for accepted connections (default 10)."""
 
-    version = 'CherryPy/' + cp_version
+    version = 'Cheroot/' + __version__
     """A version string for the HTTPServer."""
 
     software = None
     """The value to set for the SERVER_SOFTWARE entry in the WSGI environ.
 
-    If None, this defaults to ``'%s Server' % self.version``."""
+    If None, this defaults to ``'%s Server' % self.version``.
+    """
 
     ready = False
-    """An internal flag which marks whether the socket is accepting
-    connections.
-    """
+    """An internal flag which marks whether the socket is accepting connections."""
 
     max_request_header_size = 0
     """The maximum size, in bytes, for request headers, or 0 for no limit."""
@@ -1158,7 +1129,8 @@ class HTTPServer(object):
     ssl_adapter = None
     """An instance of ssl.Adapter (or a subclass).
 
-    You must have the corresponding SSL driver library installed."""
+    You must have the corresponding SSL driver library installed.
+    """
 
     def __init__(self, bind_addr, gateway, minthreads=10, maxthreads=-1,
                  server_name=None):
@@ -1381,8 +1353,7 @@ class HTTPServer(object):
         # If listening on the IPV6 any address ('::' = IN6ADDR_ANY),
         # activate dual-stack. See
         # https://github.com/cherrypy/cherrypy/issues/871.
-        if (hasattr(socket, 'AF_INET6') and family == socket.AF_INET6
-                and host in ('::', '::0', '::0.0.0.0')):
+        if hasattr(socket, 'AF_INET6') and family == socket.AF_INET6 and host in ('::', '::0', '::0.0.0.0'):
             try:
                 self.socket.setsockopt(
                     socket.IPPROTO_IPV6, socket.IPV6_V6ONLY, 0)
@@ -1546,9 +1517,7 @@ class HTTPServer(object):
 
 
 class Gateway(object):
-
-    """A base class to interface HTTPServer with other systems, such as WSGI.
-    """
+    """A base class to interface HTTPServer with other systems, such as WSGI."""
 
     def __init__(self, req):
         self.req = req
