@@ -6,14 +6,14 @@ import six
 
 import nose
 
-from chroot import _compat as compat
+from cheroot import _compat as compat
 
 
 class StringTester(unittest.TestCase):
 
     def test_ntob_non_native(self):
-        """
-        ntob should raise an Exception on unicode.
+        """ntob should raise an Exception on unicode.
+
         (Python 2 only)
 
         See #1132 for discussion.
